@@ -255,6 +255,7 @@ mddev_dev_t load_partitions(void)
 }
 
 struct createinfo createinfo = {
+	.symlinks = 1,
 #ifdef DEBIAN
 	.gid = 6, /* disk */
 	.mode = 0660,
@@ -290,6 +291,8 @@ int parse_auto(char *str, char *msg, int config)
 		len = e - str;
 		if ((len == 2 && strncasecmp(str,"md",2)==0)) {
 			autof = config ? 5 : 3;
+		} else if ((len == 3 && strncasecmp(str,"yes",3)==0)) {
+			autof = 2;
 		} else if ((len == 3 && strncasecmp(str,"mdp",3)==0)) {
 			autof = config ? 6 : 4;
 		} else if ((len == 1 && strncasecmp(str,"p",1)==0) ||
@@ -364,8 +367,11 @@ static void createline(char *line)
 			if (!createinfo.supertype)
 				fprintf(stderr, Name ": metadata format %s unknown, ignoring\n",
 					w+9);
-
-		} else {
+		} else if (strncasecmp(w, "symlinks=yes", 12) == 0)
+			createinfo.symlinks = 1;
+		else if  (strncasecmp(w, "symlinks=no", 11) == 0)
+			createinfo.symlinks = 0;
+		else {
 			fprintf(stderr, Name ": unrecognised word on CREATE line: %s\n",
 				w);
 		}
@@ -458,7 +464,7 @@ void arrayline(char *line)
 				fprintf(stderr, Name ": only specify bitmap file once. %s ignored\n",
 					w);
 			else
-				mis.bitmap_file = w+7;
+				mis.bitmap_file = strdup(w+7);
 
 		} else if (strncasecmp(w, "devices=", 8 ) == 0 ) {
 			if (mis.devices)
